@@ -19,8 +19,12 @@ from zarr.core.array import (
     _parse_chunk_key_encoding,
 )
 from zarr.core.chunk_grids import RegularChunkGrid, _auto_partition
-from zarr.core.common import JSON, DimensionNames, parse_dtype, parse_shapelike
+from zarr.core.common import JSON, DimensionNames, parse_shapelike
 from zarr.core.config import config as zarr_config
+from zarr.core.dtype import (
+    get_data_type_from_native_dtype,
+)
+from zarr.core.dtype.common import HasItemSize
 from zarr.core.metadata.v2 import ArrayV2Metadata
 from zarr.core.metadata.v3 import ArrayV3Metadata
 from zarr.core.sync import sync
@@ -36,6 +40,7 @@ if TYPE_CHECKING:
     from zarr.core.array import CompressorsLike, FiltersLike, SerializerLike, ShardsLike
     from zarr.core.chunk_key_encodings import ChunkKeyEncoding, ChunkKeyEncodingLike
     from zarr.core.common import ChunkCoords, MemoryOrder, ShapeLike, ZarrFormat
+    from zarr.core.dtype.wrapper import ZDType
 
 
 async def parse_store(
@@ -265,7 +270,7 @@ def create_array_metadata(
     filters: FiltersLike = "auto",
     compressors: CompressorsLike = "auto",
     serializer: SerializerLike = "auto",
-    fill_value: Any | None = None,
+    fill_value: Any = 0,
     order: MemoryOrder | None = None,
     zarr_format: ZarrFormat,
     attributes: dict[str, JSON] | None = None,
@@ -275,14 +280,19 @@ def create_array_metadata(
     """
     Create array metadata
     """
-    dtype_parsed = parse_dtype(dtype, zarr_format=zarr_format)
+    dtype_parsed = get_data_type_from_native_dtype(dtype)
     shape_parsed = parse_shapelike(shape)
     chunk_key_encoding_parsed = _parse_chunk_key_encoding(
         chunk_key_encoding, zarr_format=zarr_format
     )
-
+    item_size = 1
+    if isinstance(dtype_parsed, HasItemSize):
+        item_size = dtype_parsed.item_size
     shard_shape_parsed, chunk_shape_parsed = _auto_partition(
-        array_shape=shape_parsed, shard_shape=shards, chunk_shape=chunks, dtype=dtype_parsed
+        array_shape=shape_parsed,
+        shard_shape=shards,
+        chunk_shape=chunks,
+        item_size=item_size,
     )
 
     if order is None:
@@ -293,11 +303,11 @@ def create_array_metadata(
 
     if zarr_format == 2:
         filters_parsed, compressor_parsed = _parse_chunk_encoding_v2(
-            compressor=compressors, filters=filters, dtype=np.dtype(dtype)
+            compressor=compressors, filters=filters, dtype=dtype_parsed
         )
         return ArrayV2Metadata(
             shape=shape_parsed,
-            dtype=np.dtype(dtype),
+            dtype=dtype_parsed,
             chunks=chunk_shape_parsed,
             order=order_parsed,
             dimension_separator=chunk_key_encoding_parsed.separator,
@@ -398,7 +408,7 @@ def meta_from_array(
     filters: FiltersLike = "auto",
     compressors: CompressorsLike = "auto",
     serializer: SerializerLike = "auto",
-    fill_value: Any | None = None,
+    fill_value: Any = 0,
     order: MemoryOrder | None = None,
     zarr_format: ZarrFormat = 3,
     attributes: dict[str, JSON] | None = None,
@@ -423,3 +433,12 @@ def meta_from_array(
         chunk_key_encoding=chunk_key_encoding,
         dimension_names=dimension_names,
     )
+
+
+def skip_object_dtype(dtype: ZDType[Any, Any]) -> None:
+    if dtype.dtype_cls is type(np.dtype("O")):
+        msg = (
+            f"{dtype} uses the numpy object data type, which is not a valid target for data "
+            "type resolution"
+        )
+        pytest.skip(msg)
