@@ -18,6 +18,7 @@ from zarr.api.asynchronous import (
     open_consolidated,
 )
 from zarr.core.buffer import cpu, default_buffer_prototype
+from zarr.core.dtype import parse_data_type
 from zarr.core.group import ConsolidatedMetadata, GroupMetadata
 from zarr.core.metadata import ArrayV3Metadata
 from zarr.core.metadata.v2 import ArrayV2Metadata
@@ -503,7 +504,7 @@ class TestConsolidated:
     async def test_consolidated_metadata_v2(self):
         store = zarr.storage.MemoryStore()
         g = await AsyncGroup.from_store(store, attributes={"key": "root"}, zarr_format=2)
-        dtype = "uint8"
+        dtype = parse_data_type("uint8", zarr_format=2)
         await g.create_array(name="a", shape=(1,), attributes={"key": "a"}, dtype=dtype)
         g1 = await g.create_group(name="g1", attributes={"key": "g1"})
         await g1.create_group(name="g2", attributes={"key": "g2"})
@@ -624,7 +625,6 @@ async def test_consolidated_metadata_encodes_special_chars(
     memory_store: Store, zarr_format: ZarrFormat, fill_value: float
 ):
     root = await group(store=memory_store, zarr_format=zarr_format)
-    _child = await root.create_group("child", attributes={"test": fill_value})
     _time = await root.create_array("time", shape=(12,), dtype=np.float64, fill_value=fill_value)
     await zarr.api.asynchronous.consolidate_metadata(memory_store)
 
@@ -638,18 +638,11 @@ async def test_consolidated_metadata_encodes_special_chars(
             "consolidated_metadata"
         ]["metadata"]
 
-    if np.isnan(fill_value):
-        expected_fill_value = "NaN"
-    elif np.isneginf(fill_value):
-        expected_fill_value = "-Infinity"
-    elif np.isinf(fill_value):
-        expected_fill_value = "Infinity"
+    expected_fill_value = _time._zdtype.to_json_scalar(fill_value, zarr_format=2)
 
     if zarr_format == 2:
-        assert root_metadata["child/.zattrs"]["test"] == expected_fill_value
         assert root_metadata["time/.zarray"]["fill_value"] == expected_fill_value
     elif zarr_format == 3:
-        assert root_metadata["child"]["attributes"]["test"] == expected_fill_value
         assert root_metadata["time"]["fill_value"] == expected_fill_value
 
 
